import { ARIARoleDefinitionKey } from 'aria-query';
import { IsInaccessibleOptions } from 'dom-accessibility-api';
export { isSubtreeInaccessible } from 'dom-accessibility-api';

declare class StaticText {
    text: string;
    constructor(text: string);
    toString(): string;
}

type AsNonLandmarkRoles = 'HeaderAsNonLandmark' | 'FooterAsNonLandmark';
type VirtualRoles = 'Abbr' | 'Audio' | 'Canvas' | 'Details' | 'DescriptionListDetails' | 'DescriptionList' | 'DescriptionListTerm' | 'DisclosureTriangle' | 'EmbeddedObject' | 'Figcaption' | 'PluginObject' | 'LabelText' | 'LineBreak' | 'Video';
type ARIARoleDefinitionKeyExtended = ARIARoleDefinitionKey | AsNonLandmarkRoles | VirtualRoles;
type TextMatcherFunction = (content: string, element: Element | null) => boolean;
type TextMatcher = string | number | RegExp | TextMatcherFunction;
type A11yTreeNodeContext = {
    isListSubtree?: boolean;
    isNonLandmarkSubtree?: boolean;
    isInaccessibleOptions?: IsInaccessibleOptions;
};
type A11yTreeNodeState = {
    busy: boolean;
    checked?: boolean;
    current: string | boolean;
    disabled: boolean;
    expanded?: boolean;
    pressed?: boolean;
    selected?: boolean;
};
type A11yTreeNodeQueries = {
    level?: number;
    value: {
        min?: number;
        max?: number;
        now?: number;
        text?: string;
    };
};
type A11yTreeNode = {
    element: HTMLElement;
    role: HTMLElement['role'] | undefined;
    name: string;
    description: string;
    state: A11yTreeNodeState;
    queries: A11yTreeNodeQueries;
    children?: (A11yTreeNode | StaticText)[];
};
type A11yTreeNodeStateMatch = Partial<A11yTreeNodeState>;
type A11yTreeNodeQueriesMatch = {
    level?: number;
    value?: {
        min?: number;
        max?: number;
        now?: number;
        text?: TextMatcher;
    };
};
type A11yTreeNodeMatch = {
    role?: HTMLElement['role'];
    name?: TextMatcher;
    description?: TextMatcher;
    state?: A11yTreeNodeStateMatch;
    queries?: A11yTreeNodeQueriesMatch;
    children?: (A11yTreeNodeMatch | string | RegExp)[];
};

type MatcherOptions = A11yTreeNodeContext;

declare global {
  namespace jest {
    interface Matchers<R> {
      /**
       * @description
       * Asserts that the accessibility tree for an element matches the expected structure.
       * An accessibility tree represents how a user agent (such as a screen reader) processes and communicates
       * accessibility information from the DOM. Use this matcher to check whether the pertinent accessibility
       * properties and relationships are correctly established, ensuring an accessible experience for users
       * of assistive technologies.
       *
       * The expected structure is defined by the `A11yTreeNodeMatch` interface, which can accommodate various
       * properties like role, name, description, state, and children.
       *
       * The second parameter allows for optional configuration settings
       *
       * @example
       * // Example with byRole hierarchy:
       * <nav aria-label="Main navigation">
       *   <ul>
       *     <li><a href="/home">Home</a></li>
       *     <li><a href="/about">About</a></li>
       *   </ul>
       *   <button disabled>Click me</button>
       * </nav>
       *
       * expect(screen.getByRole('navigation', { name: 'Main navigation' })).toHaveA11yTree(
       *   byRole('navigation', [
       *     byRole('list', [
       *       byRole('listitem', [
       *         byRole('link', { name: 'Home' }),
       *       ]),
       *       byRole('listitem', [
       *         byRole('link', { name: 'About' }),
       *       ]),
       *     ]),
       *     byRole('button', { name: 'Click me', disabled: true }),
       *   ])
       * );
       *
       * @see
       * - [W3C Accessibility Tree](https://www.w3.org/TR/wai-aria/#accessibility_tree)
       * - [ARIA in HTML](https://www.w3.org/TR/html-aria/)
       */
      toHaveA11yTree(match?: A11yTreeNodeMatch, options?: MatcherOptions): R;
    }
  }
}

type NameDescriptionStateQueries = Omit<A11yTreeNodeMatch, 'role' | 'children'> & A11yTreeNodeStateMatch & A11yTreeNodeQueriesMatch;
declare function byRole(role: A11yTreeNodeMatch['role'], properties: NameDescriptionStateQueries): A11yTreeNodeMatch;
declare function byRole(role: A11yTreeNodeMatch['role'], properties: NameDescriptionStateQueries, children: A11yTreeNodeMatch['children']): A11yTreeNodeMatch;
declare function byRole(role: A11yTreeNodeMatch['role'], children: A11yTreeNodeMatch['children']): A11yTreeNodeMatch;
declare function byRole(role: A11yTreeNodeMatch['role'], name: string | RegExp): A11yTreeNodeMatch;
declare function byRole(role: A11yTreeNodeMatch['role'], name: string | RegExp, children: A11yTreeNodeMatch['children']): A11yTreeNodeMatch;
declare function byRole(role: A11yTreeNodeMatch['role']): A11yTreeNodeMatch;

declare const getAccessibilityTree: (element: HTMLElement, { isListSubtree: userListSubtree, isNonLandmarkSubtree: userNonLandmarkSubtree, isInaccessibleOptions, }?: MatcherOptions) => A11yTreeNode | null;

/**
 * Removes container nodes from the tree, flattening the tree.
 */
declare const pruneContainerNodes: (node: A11yTreeNode) => A11yTreeNode;

type Config = {
    isInaccessibleOptions?: IsInaccessibleOptions;
};
declare const config: Config;
declare const getConfig: () => typeof config;
declare const configToolkit: (options: Partial<Config>) => void;

export { A11yTreeNode, A11yTreeNodeMatch, A11yTreeNodeState, ARIARoleDefinitionKeyExtended, AsNonLandmarkRoles, MatcherOptions, VirtualRoles, byRole, configToolkit, getAccessibilityTree, getConfig, pruneContainerNodes };
